<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;

class RelRentabiProduto  {

    protected $RelRentabiProdutoService;
    protected $utilService;
    
    public function __construct(Services\RelRentabiProdutoService $RelRentabiProdutoService, Utilities\Util $utilService) {
        $this->RelRentabiProdutoService = $RelRentabiProdutoService;
        $this->utilService = $utilService;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        set_time_limit(120);
        $conditions = $this->utilService->getParamsAsArray($request);
        $conditions = $this->buildParamsFaturamento($conditions);
        $result = [];
        
        if(isset($conditions['CDTIPOCUSTO']) && !empty($conditions['CDTIPOCUSTO'])) {
            $result = $this->RelRentabiProdutoService->find($conditions['CDTIPOCUSTO'], $conditions['CDFILIAL'], $conditions['TCDFILIAL'], $conditions['NRORG'], $conditions['CDLOJA'], $conditions['TCDLOJA'], $conditions['DTVENDAINI'], $conditions['DTVENDAFIN'], $conditions['PRODNIV'], $conditions['TCDPRODUTO'], $conditions['CDPRODLIST'], $conditions['CDARVPRODF'], $conditions['CDARVPRODI'], $conditions['CDNVPRODUTOAGRUP'], $conditions['FLAGIMPOSTO']);
    
            $response->addDataSet(new DataSet("vnd0160E_RelRentabProduto", $result));
            
            return $response;
        }
        
        return $result;
        
    }
    
    public function buildParamsFaturamento($conditions){
        
        $result = [];
        
        if(isset($conditions['CDFILIAL']) && !empty($conditions['CDFILIAL'])) {
            
            $conditions['CDFILIAL'] = implode('_', $conditions['CDFILIAL']);
            $conditions['TCDFILIAL'] = 'F';
            
            if(!array_key_exists('CDLOJA', $conditions)) {
                $conditions['CDLOJA'] = '';
                $conditions['TCDLOJA'] = 'T';
            } else {
                $conditions['CDLOJA'] = implode('_', $conditions['CDLOJA']);
                $conditions['TCDLOJA'] = '';
            }
            if(array_key_exists('DTENTRVENDA', $conditions)) {
                $conditions['DTVENDAINI'] = $conditions['DTENTRVENDA'][0];
                $conditions['DTVENDAFIN'] = $conditions['DTENTRVENDA'][1];
                unset($conditions['DTENTRVENDA']);
            }
            if (isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'I'){
               if (empty($conditions['CDARVPRODF'])){
                    $conditions['CDARVPRODF'] = $conditions['CDARVPRODI'];
               }
               $conditions['TCDPRODUTO'] = null;
               $conditions['CDPRODLIST'] = null;
               $conditions['PRODNIV'] = null;
            }elseif (isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'D') {
               $conditions['PRODNIV'] = str_replace('%', '', str_replace('.', '', $conditions['PRODNIV'])).'%';
               $conditions['TCDPRODUTO'] = null;
               $conditions['CDPRODLIST'] = null;
               $conditions['CDARVPRODF'] = null;
               $conditions['CDARVPRODI'] = null;
            }
            elseif (isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'L'){
               $conditions['CDPRODLIST'] = implode('_', $conditions['CDPRODUTO']);
               $conditions['TCDPRODUTO'] = null;
               $conditions['CDARVPRODF'] = null;
               $conditions['CDARVPRODI'] = null;
               $conditions['PRODNIV'] = null;
            }
            else{
               $conditions['TCDPRODUTO'] = 'T';
               $conditions['CDPRODLIST'] = null;
               $conditions['CDARVPRODF'] = null;
               $conditions['CDARVPRODI'] = null;
               $conditions['PRODNIV'] = null;
            }
            return $conditions;
        }
        return $result;
    }
}
