<?php 

namespace Teknisa\MAN\Factories\OMClient;

use Teknisa\MAN\Utilities\cURLHelper;

class OMClientRepo extends AOMClient implements IClientRepo {
    const GET_ORGANIZATION_ROUTE      = "/api/findOrganizationInformation";

    const FAILED_TO_CALL_OM    = "There was a problem while trying to connect to OM! Try again later: ";
    const NO_ORG_FOUND_MESSAGE = "The organization was not found with the given NRORG.";
    
    public function __construct($projectId) {
        parent::__construct();
        $this->projectId = $projectId;
    }
    
    public function getOrganizationByNRORG($nrorg) {
        if(empty($this->getRequestToken())) {
            $this->setRequestToken();
        }    
        
        $url = $this->omServiceUrl . self::GET_ORGANIZATION_ROUTE; 
        $postData = array(
            'requestType' => 'FilterData',
            'disableLoader' => false,
            'filter' => array(
                array(
                    'name' => 'NRORG', 
                    'operator' => '=', 
                    'value' => $nrorg
                ),
                array(
                    'name' => 'PRODUCT_ID', 
                    'operator' => '=', 
                    'value' => $this->projectId
                )
            ),
        );
        
        try {
            $orgs = $this->callOMAPI($url, self::GET_ORGANIZATION_ROUTE, $postData, 'organizationInformation');
            $org = empty($orgs) ? null : $orgs[0];
    
            if(empty($org)) {
                throw new \Exception(self::NO_ORG_FOUND_MESSAGE);
            }
            
            return $org; // Client Type
        } catch(\Exception $e) {
            throw new \Exception(self::FAILED_TO_CALL_OM . $e->getMessage());
        }
    }
}