<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;

use Teknisa\MAN\Factories\Retail\TabelaPreco as TabelaPrecoFactory;
use Teknisa\MAN\Factories\Retail\Produto as ProdutoFactory;
use Teknisa\MAN\Factories\Retail\CustoComposicaoProduto as CustoComposicaoProdutoFactory;
use Teknisa\MAN\Factories\Retail\ImpostosItemPreco as ImpostosItemPrecoFactory;

class CalcularPrecos {
    
    private $pricingTabFactory;
    private $productFactory;
    private $custoCompProdFactory;
    
    private $NRORG;
    
    const INTERVAL_PRODUCTS_FILTER = 'I';
    const LIST_PRODUCTS_FILTER     = 'L';
    
    public function __construct(TabelaPrecoFactory $pricingTabFactory, 
                                ProdutoFactory $productFactory,  
                                CustoComposicaoProdutoFactory $custoCompProdFactory, 
                                ImpostosItemPrecoFactory $impostosItemPrecoFactory, 
                                $environment) {
        $this->pricingTabFactory        = $pricingTabFactory;
        $this->productFactory           = $productFactory;
        $this->custoCompProdFactory     = $custoCompProdFactory;
        $this->impostosItemPrecoFactory = $impostosItemPrecoFactory;
        
        $this->NRORG                    = $environment->getNrOrgTrab();
    }

    private function getFilteredProductsByUser($row) {
        $pricingTableFilters = array(
            'CDFILIAL'      => $row['CDFILIAL'],
            'CDTABEPREC'    => $row['CDTABEPREC'],
            'DTINIVGPREC'   => $row['DTINIVGPREC'],
            'NRORG'         => $row['NRORG'],
        );
        
        if($row['TIPOPROD'] === self::INTERVAL_PRODUCTS_FILTER) {
            $productsFilters = array(
                'PRODUTOS' => null,     
                'CDARVPRODINI' => $row['CDARVPRODINI'],
                'CDARVPRODFIN' => $row['CDARVPRODFIN'],
            );
        } else if($row['TIPOPROD'] === self::LIST_PRODUCTS_FILTER) {
            $productsFilters = array(
                'PRODUTOS'     => implode('_', $row['PRODUTOS']),
                'CDARVPRODINI' => null,
                'CDARVPRODFIN' => null,
            );
        }
        
        return $this->pricingTabFactory->getProductsPrice($pricingTableFilters, $productsFilters);
    }

    public function calcularPreco($row) {
        $pricingTableProducts = $this->getFilteredProductsByUser($row);
        $result = [];

        foreach($pricingTableProducts as $product) {
            try {
                // Buscar percentual para cálculo da tabela PERCACRPROD:
                $PERCENT = $this->productFactory->getVRPERCACRPROD($row['CDFILIAL'], $product['CDPRODUTO']);
                
                
                // Buscar o custo da composição dos produtos.  
                $doCusto = $this->custoCompProdFactory->getCustoCompProd(
                                    $row['CDFILIAL'], $product['CDPRODUTO'], $row['CDTIPOCUSTO']);

                if ($row["IDCONSIDERAIMPOSTO"] === 'S') {
                    $totalImposto = $this->impostosItemPrecoFactory->getTotalImpostos(
                                                $row['CDFILIAL'], $product['CDARVPROD'],  $row['NRORG']);

                    $doPercTotal = (100 - $totalImposto + $PERCENT) / 100;
                } else {
                    $doPercTotal = (100 + $PERCENT) / 100;
                }

                if($doPercTotal === 0.00) {
                    $doPercTotal = 1;
                }

                $VRCUSTITEM = 0.00;

                if ($row["IDCONSIDERAIMPOSTO"] === 'S') {
                    $VRCUSTITEM = ($doCusto * $doPercTotal);
                } else {
                    $VRCUSTITEM = ($doCusto / $doPercTotal);
                }
                
                $VRMKUPITEM = $doPercTotal;
                
                $this->pricingTabFactory->updateMarkupCols(
                    $row,
                    $product['CDPRODUTO'],
                    $VRCUSTITEM,
                    $VRMKUPITEM
                );
                
                array_push($result, array(
                    'CDPRODUTO' => $product['CDPRODUTO'],
                    'STATUS'    => 'SUCCESS'
                ));
            } catch (\Exception $e) {
                var_dump($e->getMessage());
                
                array_push($result, array(
                    'CDPRODUTO' => $product['CDPRODUTO'],
                    'STATUS'    => 'ERROR',
                    'STACK'     => $e->getMessage(),
                ));
            }
        }
        
        return $result;
    }
}
